==> smart_tab_layout-1.0.0.tar.gz: lib/src/smart_image_tab_indicator.dart <==
import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:smart_tab_layout/src/smart_underline_tab_indicator.dart';

/// 图片指示器
///
/// [SmartImageTabIndicator] Picture indicator
/// [SmartUnderlineTabIndicator] Underline indicator
///
class SmartImageTabIndicator extends Decoration {
  const SmartImageTabIndicator({
    this.direction = Axis.horizontal,
    this.image,
    this.insets = EdgeInsets.zero,
  }) : assert(insets != null);

  final ui.Image? image;

  final Axis direction;

  /// Locates the selected tab's underline relative to the tab's boundary.
  ///
  /// The [TabBar.indicatorSize] property can be used to define the tab
  /// indicator's bounds in terms of its (centered) tab widget with
  /// [TabBarIndicatorSize.label], or the entire tab with
  /// [TabBarIndicatorSize.tab].
  final EdgeInsetsGeometry insets;

  @override
  _UnderlinePainter createBoxPainter([VoidCallback? onChanged]) {
    return _UnderlinePainter(this, image, direction, onChanged);
  }

  Rect _indicatorRectFor(Rect rect, TextDirection textDirection) {
    assert(rect != null);
    assert(textDirection != null);
    final Rect indicator = insets.resolve(textDirection).deflateRect(rect);
    if (direction == Axis.horizontal) {
      return Rect.fromLTWH(
        indicator.left,
        indicator.bottom - (image?.height ?? 0),
        indicator.width,
        (image?.height ?? 0).toDouble(),
      );
    } else {
      return Rect.fromLTWH(
        indicator.right - (image?.width ?? 0),
        indicator.top,
        (image?.width ?? 0).toDouble(),
        indicator.height,
      );
    }
  }

  @override
  Path getClipPath(Rect rect, TextDirection textDirection) {
    return Path()..addRect(_indicatorRectFor(rect, textDirection));
  }

  static Future<ui.Image> load(String asset, Size size) async {
    ByteData data = await rootBundle.load(asset);
    ui.Codec codec = await ui.instantiateImageCodec(
      data.buffer.asUint8List(),
      targetWidth: size.width.toInt(),
      targetHeight: size.height.toInt(),
    );
    ui.FrameInfo fi = await codec.getNextFrame();
    return fi.image;
  }
}

class _UnderlinePainter extends BoxPainter {
  _UnderlinePainter(
      this.decoration, this.image, this.direction, VoidCallback? onChanged)
      : assert(decoration != null),
        super(onChanged);

  final ui.Image? image;
  final Axis direction;
  final SmartImageTabIndicator decoration;

  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration configuration) {
    assert(configuration != null);
    assert(configuration.size != null);
    final Rect rect = offset & configuration.size!;
    final TextDirection textDirection = configuration.textDirection!;
    final Rect indicator = decoration._indicatorRectFor(rect, textDirection);
    final Paint paint = Paint();
    if (image != null) {
      if (direction == Axis.horizontal) {
        double width = image!.width.toDouble() / 2;
        canvas.drawImage(image!, indicator.topCenter.translate(-width, 0), paint);
      } else {
        double height = image!.height.toDouble() / 2;
        canvas.drawImage(image!, indicator.centerLeft.translate(0, -height), paint);
      }
    }
  }
}

==> smart_tab_layout-1.0.7.tar.gz: lib/src/smart_image_tab_indicator.dart <==
import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:smart_tab_layout/src/smart_underline_tab_indicator.dart';

typedef CustomDraw = void Function(
    Canvas canvas, Offset offset, ImageConfiguration configuration, Rect rect);

/// 图片指示器
///
/// [SmartImageTabIndicator] Picture indicator
/// [SmartUnderlineTabIndicator] Underline indicator
///
class SmartImageTabIndicator extends Decoration {
  const SmartImageTabIndicator({
    this.direction = Axis.horizontal,
    this.image,
    this.insets = EdgeInsets.zero,
    this.customDraw,
  }) : assert(insets != null);

  final ui.Image? image;

  final Axis direction;

  /// Locates the selected tab's underline relative to the tab's boundary.
  ///
  /// The [TabBar.indicatorSize] property can be used to define the tab
  /// indicator's bounds in terms of its (centered) tab widget with
  /// [TabBarIndicatorSize.label], or the entire tab with
  /// [TabBarIndicatorSize.tab].
  final EdgeInsetsGeometry insets;

  final CustomDraw? customDraw;

  @override
  _UnderlinePainter createBoxPainter([VoidCallback? onChanged]) {
    return _UnderlinePainter(
        this, image, direction, this.customDraw, onChanged);
  }

  Rect _indicatorRectFor(Rect rect, TextDirection textDirection) {
    assert(rect != null);
    assert(textDirection != null);
    final Rect indicator = insets.resolve(textDirection).deflateRect(rect);
    if (direction == Axis.horizontal) {
      return Rect.fromLTWH(
        indicator.left,
        indicator.bottom - (image?.height ?? 0),
        indicator.width,
        (image?.height ?? 0).toDouble(),
      );
    } else {
      return Rect.fromLTWH(
        indicator.right - (image?.width ?? 0),
        indicator.top,
        (image?.width ?? 0).toDouble(),
        indicator.height,
      );
    }
  }

  @override
  Path getClipPath(Rect rect, TextDirection textDirection) {
    return Path()..addRect(_indicatorRectFor(rect, textDirection));
  }

  static Future<ui.Image> load(String asset, Size size) async {
    ByteData data = await rootBundle.load(asset);
    ui.Codec codec = await ui.instantiateImageCodec(
      data.buffer.asUint8List(),
      targetWidth: size.width.toInt(),
      targetHeight: size.height.toInt(),
    );
    ui.FrameInfo fi = await codec.getNextFrame();
    return fi.image;
  }
}

class _UnderlinePainter extends BoxPainter {
  _UnderlinePainter(this.decoration, this.image, this.direction,
      this.customDraw, VoidCallback? onChanged)
      : assert(decoration != null),
        super(onChanged);

  final ui.Image? image;
  final Axis direction;
  final SmartImageTabIndicator decoration;
  final CustomDraw? customDraw;

  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration configuration) {
    assert(configuration != null);
    assert(configuration.size != null);
    final Rect rect = offset & configuration.size!;
    final TextDirection textDirection = configuration.textDirection!;
    final Rect indicator = decoration._indicatorRectFor(rect, textDirection);
    final Paint paint = Paint();
    customDraw?.call(canvas, offset, configuration, indicator);
    if (image != null) {
      if (direction == Axis.horizontal) {
        double width = image!.width.toDouble() / 2;
        canvas.drawImage(
            image!, indicator.topCenter.translate(-width, 0), paint);
      } else {
        double height = image!.height.toDouble() / 2;
        canvas.drawImage(
            image!, indicator.centerLeft.translate(0, -height), paint);
      }
    }
  }
}
